#ifndef DAEMON_NATIVE_EVENTS_CAPTURER_H_
#define DAEMON_NATIVE_EVENTS_CAPTURER_H_

#include <systemd/sd-login.h>
#include <napi.h>
#include <poll.h>
#include <assert.h>
#include <unistd.h>
#include <memory>
#include <thread>
#include <iostream>
#include <chrono>

#include "./LoginStateEvents.h"
#include "./KeyPressEvents.h"
#include "../constants/common.h"

using namespace std;
using namespace Napi;
using namespace LoginEvents;
using namespace KeyboardEvents;

namespace DaemonNativeEvents
{
    class DaemonNativeEventsCapturer : public Napi::ObjectWrap<DaemonNativeEventsCapturer> {
        public:
            static Napi::Object Init(Napi::Env env, Napi::Object exports);
            DaemonNativeEventsCapturer(const Napi::CallbackInfo &info);
            ~DaemonNativeEventsCapturer();

            void ExportedStartListeners(const Napi::CallbackInfo &info);
            void CreateNativeThread();
            void ExportedcloseListeners(const Napi::CallbackInfo &info);

            Napi::Value exported_init_sd_login_events(const Napi::CallbackInfo &info);
            void exported_deinit_sd_login_events(const Napi::CallbackInfo& info);
            Napi::Value ExportedGetUsersStateLoginData(const Napi::CallbackInfo& info);

            Napi::Value exported_init_key_press_events(const Napi::CallbackInfo &info);
            void exported_deinit_key_press_events(const Napi::CallbackInfo& info);

            void get_sd_user_details();
            void get_paste_event_details();

        private:
            static Napi::FunctionReference constructor;

            std::thread nativeThread;
            Napi::ThreadSafeFunction tsfn;

            unsigned int _loopCounter;
            bool isThreadCreated;
            bool _isListenSdLoginMonitorEvents;
            bool _isListenKeyPressTrackerEvents;

            LoginStateEvents loginEventsUtility;
            KeyPressEvents keyPressEventsTrackerUtility;
    };
} // namespace DaemonNativeEvents

#endif // DAEMON_NATIVE_EVENTS_CAPTURER_H_

