export declare const authenticationUtility: IAuthenticationUtility;
export interface IAuthenticationUtility {
    /**
     * This Function takes the username, password and authenticate it with system username and password
     *  and gives true or false
     * @param username is string which we pass into this function.
     * @param password is string which we pass into this function.
     */
    authenticateUser(username: string, password: string): boolean;
}
/**
 * LoginEventsUtility will emit events on various user login events which will be used by the application
 * keyboardEventsUtility will emit events on keyboard key press
 * This utility is exposed by another .node file daemonNativeEventsUtility.node
 */
export declare const addon: any;
export declare const daemonNativeEventsCapturer: IDaemonNativeEventsCapturer;
interface IDaemonNativeEventsCapturer {
    /**
     * Created a thread for tsfn created and start listening to required events
     * @param func Takes a function (an emitter function) and defines a thread-safe function for C to pass events to node.js
     * It monitors user events like login/logout/switch user, clipboard events, screen lock detect,
     * and places a non-blocking call of the tsfn created
     */
    startListeners(func: Function): void;
    /**
     * start listening to sd login events
     * @returns bool indicating whether sd login events can be captured or not
     */
    initSdLoginEvents(): boolean;
    /**
     * stop listening to sd login events
     */
    deInitSdLoginEvents(): void;
    /**
     * Stops listening to all the events and releases tsfn
     */
    closeListeners(): void;
    /**
     * Gives current active user data
     */
    getUsersLoginData(): IUsersLoginData;
    /**
     * start listening to key press events
     */
    initKeyPressEvents(): boolean;
    /**
     * stop listening to key press events
     */
    deInitKeyPressEvents(): boolean;
}
/**
 * keyboardEventsUtility will emit events on keyboard key press
 * This utility is exposed by another .node file keyboardEventsUtility.node
 */
interface IUserSessionInfo {
    session: string;
    tty: string;
    seat: string;
    service: string;
    type: string;
    sessClass: string;
    sessState: string;
}
export interface IUserStateInfo {
    uid: number;
    state: string;
    userSessionInfo: IUserSessionInfo[];
}
export interface IUsersLoginData {
    userData: IUserStateInfo[];
}
export {};
//# sourceMappingURL=index.d.ts.map