#include <string>
#include <vector>
using namespace std;

const std::string LOCK_SCREEN_STATUS      = "lock-screen-data";
const std::string SCREEN_RESOLUTION_DATA  = "resolution-data";
const std::string PRIMARY_MONITOR_CHANGE  = "primary-monitor-change";
const std::string CURSOR_DATA             = "cursor-data";
const std::string CLIPBOARD_CHANGE        = "copy-event";

const std::string ICE_STATE_CHANGE        = "ice-state-change";
const std::string QUIC_MESSAGE_RECEIVED   = "quic-message-received";
const std::string PRE_SWITCH_P2P          = "pre-switch-p2p";
const std::string SWITCH_TO_P2P           = "switch-to-p2p";
const std::string REINIT_ON_PINGPONG_FAIL = "reinit-on-pingpong-fail";
const std::string QUIC_LOCAL_PAIR_INFO    = "quic-local-pair-info";

const unsigned int GET_SCREEN_RES_DETAILS_AFTER_X_TIMES = 10;

const unsigned int SLEEP_FOR_200MS        = 200; // in milliseconds
const unsigned int SLEEP_FOR_5MS          = 5U; // in milliseconds
const unsigned int SLEEP_FOR_10MS         = 10U; // in milliseconds
const unsigned int SLEEP_FOR_50MS         = 50U; // in milliseconds
const unsigned int SLEEP_FOR_500MS        = 500U;
const unsigned int SLEEP_FOR_450MS        = 450U;

// It is not the actual max, as adding 1 with bigger numbers like 2047483648 will be more calculative
// 65,535 is 2^16 - 1. Why -1? To make it divisible by 3. Why 2^16? Max of 2 byte short int
const unsigned int RPC_MAX_LOOP_COUNTER   = 65530;

const std::string SERVER_BUS_NAME         = "/org/gnome/SessionManager";
const std::string SERVER_OBJECT_PATH_NAME = "org.freedesktop.DBus";
const vector<string> PROPERTIES_NAMES     = {"SessionIsActive", "InhibitedActions", "HibitedActions"};

const vector<std::string> OS_NAMES = {
    "ElemOS5",
    "MintC20",
    "GnomeOS",
    "Ubuntu16",
    "KDEDesktops"
};

const std::string UBUNTU16_UNITY_SIGNAL_NAMES_LOCKED   = "desktop-lock";
const std::string UBUNTU16_UNITY_SIGNAL_NAMES_UNLOCKED = "desktop-unlock";

enum OS_TYPE_FOR_DBUS_SESSION_BUS_RULE {
    ELEMENTARY5 = 0, // GNOME 40, but uses freeDesktop interface for screen lock
    MINT20_CINNAMON, // CINNAMON, uses cinnamon interface (similar to GNOME)
    // All OS(s) supported by RemotePC like Fedora 34, Ubuntu 18 20 21, Debian 9 10, Cent 7 8, RHEL 7 8, Suse Leap 15, Budgie 21
    // uses gnome.ScreenSaver interface. So same code should work with all of them.
    OS_USING_GNOME,
    UBUNTU16_UNITY,
    KDE_DESKTOPS
};

const vector<std::string> DBUS_SESSION_BUS_MATCH_RULE = {
    "type='signal',path='/org/gnome/SessionManager',interface='org.freedesktop.DBus.Properties',member='PropertiesChanged'",
    "type='signal',path='/org/cinnamon/ScreenSaver',interface='org.cinnamon.ScreenSaver',member='ActiveChanged'",
    "type='signal',path='/org/gnome/ScreenSaver',interface='org.gnome.ScreenSaver',member='ActiveChanged'",
    "type='signal',path='/com/ubuntu/Upstart',interface='com.ubuntu.Upstart0_6',member='EventEmitted'",
    "type='signal',path='/org/freedesktop/ScreenSaver',interface='org.freedesktop.ScreenSaver',member='ActiveChanged'"
};
const vector<std::string> INTERFACE_NAME = {
    "org.freedesktop.DBus.Properties",
    "org.cinnamon.ScreenSaver",
    "org.gnome.ScreenSaver",
    "com.ubuntu.Upstart0_6",
    "org.freedesktop.ScreenSaver"
};
const vector<std::string> SIGNAL_NAMES = {
    "PropertiesChanged",
    "ActiveChanged",
    "ActiveChanged",
    "EventEmitted",
    "ActiveChanged"
};

enum SCREEN_LOCK_STATUS {
    LOCKED   = 1,
    UNLOCKED = 0,
    UNKNOWN  = -1
};

struct p2p_pair_info {
	char* r0;           // pair info
	uint64_t r1;        // length of pair info
    int64_t sigHandle;  // signal Handle
    int32_t *channelNumber;
};

enum ICE_CANDIDATE_STATE {
    NEW          = 1,
    CHECKING     = 2,
    CONNECTED    = 3,
    COMPLETED    = 4,
    FAILED       = 5,
    DISCONNECTED = 6,
    CLOSED       = 7,
    INVALID      = 8,
    OTHER        = 9
};
